import React, {Component} from 'react';
import {
  Dimensions,
  Platform,
  ScrollView,
  StyleSheet,
  View,
  Text,
  Alert,
  ActivityIndicator,
  StatusBar,
} from 'react-native';
import Button from './button';
import {connect} from 'react-redux';
import CountDown from '../count-down';
import {AUDIO_RECORDING} from '../../common/constant';
import SharePreference from '../../common/SharePreference';
import moment from 'moment';
import {
  REQUEST_SUBMIT_ANSWER,
  REQUEST_UPLOAD_RECORD,
  REQUEST_COMPONENTS_KEY_PAGES,
} from '../../redux/actions';
import {request} from '../../redux/api';
import Config from '../../Config';
import RNExitApp from 'react-native-exit-app';
// Detect screen width and height
const {width, height} = Dimensions.get('window');

class OnboardingScreens extends Component {
  // Props for ScrollView component
  static defaultProps = {
    // Arrange screens horizontally
    horizontal: true,
    // Scroll exactly to the next screen, instead of continous scrolling
    pagingEnabled: true,
    // Hide all scroll indicators
    showsHorizontalScrollIndicator: false,
    showsVerticalScrollIndicator: false,
    // Do not bounce when the end is reached
    bounces: false,
    // Do not scroll to top when the status bar is tapped
    scrollsToTop: false,
    // Remove offscreen child views
    removeClippedSubviews: true,
    // Do not adjust content behind nav-, tab- or toolbars automatically
    automaticallyAdjustContentInsets: false,
    // Fisrt is screen is active
    index: 0,
  };

  state = this.initState(this.props);

  /**
   * Initialize the state
   */
  initState(props) {
    // Get the total number of slides passed as children
    const total = props.children ? props.children.length || 1 : 0,
      // Current index
      index = total > 1 ? Math.min(props.index, total - 1) : 0,
      // Current offset
      offset = width * index;

    const state = {
      total,
      index,
      offset,
      width,
      height,
      loading: false,
    };

    // Component internals as a class property,
    // and not state to avoid component re-renders when updated
    this.internals = {
      isScrolling: false,
      offset,
    };

    return state;
  }

  // UNSAFE_componentWillMount = () => {
  //   this._panResponder = PanResponder.create({
  //     onPanResponderTerminationRequest: () => false,
  //     onStartShouldSetPanResponderCapture: () => true,
  //   });
  // };

  /**
   * Scroll begin handler
   * @param {object} e native event
   */
  onScrollBegin = (e) => {
    // Update internal isScrolling state
    this.internals.isScrolling = true;
  };

  /**
   * Scroll end handler
   * @param {object} e native event
   */
  onScrollEnd = (e) => {
    // Update internal isScrolling state
    this.internals.isScrolling = false;

    // Update index
    this.updateIndex(
      e.nativeEvent.contentOffset
        ? e.nativeEvent.contentOffset.x
        : // When scrolled with .scrollTo() on Android there is no contentOffset
          e.nativeEvent.position * this.state.width,
    );
  };

  /*
   * Drag end handler
   * @param {object} e native event
   */
  onScrollEndDrag = (e) => {
    const {
        contentOffset: {x: newOffset},
      } = e.nativeEvent,
      {children} = this.props,
      {index} = this.state,
      {offset} = this.internals;
    // Update internal isScrolling state
    // if swiped right on the last slide
    // or left on the first one
    if (
      offset === newOffset &&
      (index === 0 || index === children.length - 1)
    ) {
      this.internals.isScrolling = false;
    }
  };

  /**
   * Update index after scroll
   * @param {object} offset content offset
   */
  updateIndex = (offset) => {
    const {dispatch} = this.props;
    const state = this.state,
      diff = offset - this.internals.offset,
      step = state.width;
    let index = state.index;
    // Do nothing if offset didn't change
    if (!diff) {
      return;
    }

    // Make sure index is always an integer
    index = parseInt(index + Math.round(diff / step), 10);

    // Update internal offset
    this.internals.offset = offset;
    // Update index in the state
    this.setState({
      index,
    });
    dispatch({type: REQUEST_COMPONENTS_KEY_PAGES, payload: {keyPage: index}});
    this.props.setMute();
    this.dispatchAnswer();
  };

  dispatchAnswer = () => {
    const {dispatch, submitData} = this.props;

    const arrAnswer = submitData.user_answer;
    const recordExisted = arrAnswer.find(
      (answer) =>
        answer.type === AUDIO_RECORDING && answer.user_answer.value === '',
    );
    if (recordExisted) {
      // console.log(999, 'upload', recordExisted);
      //goi action upload file record
      dispatch({type: REQUEST_UPLOAD_RECORD, payload: recordExisted});
    }
  };

  /**
   * Swipe one slide forward
   */
  swipe = () => {
    // Ignore if already scrolling or if there is less than 2 slides
    if (this.internals.isScrolling || this.state.total < 2) {
      return;
    }
    const state = this.state,
      diff = this.state.index + 1,
      x = diff * state.width,
      y = 0;
    // Call scrollTo on scrollView component to perform the swipe
    this.scrollView && this.scrollView.scrollTo({x, y, animated: true});

    // Update internal scroll state
    this.internals.isScrolling = true;

    // Trigger onScrollEnd manually on android
    if (Platform.OS === 'android') {
      setImmediate(() => {
        this.onScrollEnd({
          nativeEvent: {
            position: diff,
          },
        });
      });
    }
  };

  prev = () => {
    if (this.internals.isScrolling || this.state.total < 2) {
      return;
    }
    const state = this.state,
      diff = this.state.index - 1,
      x = diff * state.width,
      y = 0;
    // Call scrollTo on scrollView component to perform the swipe
    this.scrollView && this.scrollView.scrollTo({x, y, animated: true});

    // Update internal scroll state
    this.internals.isScrolling = true;

    // Trigger onScrollEnd manually on android
    if (Platform.OS === 'android') {
      setImmediate(() => {
        this.onScrollEnd({
          nativeEvent: {
            position: diff,
          },
        });
      });
    }
  };
  /**
   * Render ScrollView component
   * @param {array} slides to swipe through
   */
  renderScrollView = (pages) => {
    return (
      <ScrollView
        ref={(component) => {
          this.scrollView = component;
        }}
        {...this.props}
        style={{flex: 1}}
        onScrollBeginDrag={this.onScrollBegin}
        onMomentumScrollEnd={this.onScrollEnd}
        onScrollEndDrag={this.onScrollEndDrag}>
        {pages.map((page, i) => (
          // Render each slide inside a View
          <ScrollView>
            <View style={[styles.fullScreen, styles.slide]} key={i}>
              {page}
            </View>
          </ScrollView>
        ))}
      </ScrollView>
    );
  };

  /**
   * Render pagination indicators
   */
  renderPagination = () => {
    if (this.state.total <= 1) {
      return null;
    }

    const ActiveDot = <View style={[styles.dot, styles.activeDot]} />,
      Dot = <View style={styles.dot} />;

    let dots = [];

    for (let key = 0; key < this.state.total; key++) {
      dots.push(
        key === this.state.index
          ? // Active dot
            React.cloneElement(ActiveDot, {key})
          : // Other dots
            React.cloneElement(Dot, {key}),
      );
    }

    return (
      <View pointerEvents="none" style={[styles.pagination, styles.fullScreen]}>
        {dots}
      </View>
    );
  };
  renderQuestions = () => {
    const last = this.state.index + 1;
    const total = this.state.total;
    return (
      <View style={[styles.question]}>
        <Text style={styles.questionText}>
          Question {last}/{total}
        </Text>
        <View
          style={{
            width: '100%',
            height: 1,
            backgroundColor: 'gray',
            marginTop: '2%',
            paddingHorizontal: '5%',
          }}
        />
      </View>
    );
  };
  /**
   * Render Continue or Done button
   */
  renderButton = () => {
    const lastScreen = this.state.index === this.state.total - 1;
    return (
      <View pointerEvents="box-none" style={[styles.buttonWrapper]}>
        {lastScreen ? (
          // Show this button on the last screen
          // TODO: Add a handler that would send a user to your app after onboarding is complete
          <Button text="Finish" onPress={() => this.confirmAnwer(0)} />
        ) : (
          // Or this one otherwise
          <Button text="Next" onPress={() => this.swipe()} />
        )}
      </View>
    );
  };

  confirmAnwer = (type) => {
    const {user_answer} = this.props.submitData;
    const {children, data} = this.props;
    if (type === 0) {
      this.dispatchAnswer();
      Alert.alert(
        'Thông báo',
        `Bạn đã trả lời ${user_answer.length}/${children.length} câu hỏi. Sẵn sàng nộp bài?`,
        [{text: 'Huỷ bỏ'}, {text: 'Đồng ý', onPress: () => this.acceptAnwer()}],
      );
    } else {
      Alert.alert(
        'Thông báo',
        `Bạn đã hết thời gian làm bài.\nBạn đã hoàn thành ${user_answer.length}/${children.length} câu hỏi.`,
        [
          {
            text: 'Kết thúc',
            style: 'cancel',
          },
          {cancelable: false},
        ],
      );
      setTimeout(() => {
        this.acceptAnwer();
      }, 2000);
    }
  };

  acceptAnwer = () => {
    const {data, submitData} = this.props;
    let dataReview = [];
    submitData.user_answer.forEach((item) => {
      const data = {
        question_configuration: item.question_configuration,
        question_id: item.question_id,
        section_id: item.section_id,
        section_tag: item.section_tag,
        user_answer: item.user_answer,
      };
      dataReview.push(data);
    });
    // dispath action request submit answer
    const dataAnswer = {
      user_test_id: data.data.id,
      data: JSON.stringify(dataReview),
      started_at: SharePreference.startTest,
      finished_at: moment().unix(),
    };
    console.log('345', dataAnswer);
    this.postSubmitAnswer(dataAnswer);
  };

  postSubmitAnswer = (params) => {
    const {route} = this.props;
    // dispatch({type: REQUEST_SUBMIT_ANSWER, payload: params});
    // const {data, error} = this.props.submitAnswer;
    // if (data) {
    //   console.log(366, data);
    // } else {
    //   console.log(368, error);
    //   const paserData = JSON.parse(error);
    //   this.dialogNotify(paserData.message, params);
    // }
    this.setState({loading: true});
    request({
      url: `${Config.url}/api/test/submit-answers`,
      method: 'post',
      headers: {Authorization: SharePreference.token_hotqa},
      body: params,
    })
      .then((res) => {
        if (res.body.status === 200 && res.body.success) {
          Alert.alert('', 'Chúc mừng bạn đã hoàn thành xong bài kiểm tra.', [
            {
              text: 'Ok',
              onPress: () => {
                route ? route.goBack() : RNExitApp.exitApp();
              },
            },
          ]);
        } else {
          this.dialogNotify(res.body.message, params);
        }
        this.setState({loading: false});
      })
      .catch((e) => {
        const error = JSON.parse(e.message);
        this.dialogNotify(error.message, params);
        this.setState({loading: false});
      });
  };

  dialogNotify = (message, params) => {
    Alert.alert('Lỗi', message, [
      {text: 'Huỷ bỏ'},
      {text: 'Thử lại', onPress: () => this.postSubmitAnswer(params)},
    ]);
  };

  renderButtonPrev = () => {
    const lastScreen = this.state.index >= 1;
    return (
      <View pointerEvents="box-none" style={[styles.buttonPrev]}>
        {lastScreen && <Button text="Prev" onPress={() => this.prev()} />}
      </View>
    );
  };

  /**
   * Render the component
   */
  render = ({children, data} = this.props) => {
    const {data: cur_data} = data;
    return (
      <View style={[styles.container, styles.fullScreen]}>
        <StatusBar hidden={true} />
        <View style={{flex: 0.15}}>{this.renderQuestions()}</View>
        <View style={styles.countDown}>
          <CountDown
            minutes={cur_data.test_data.time_limit}
            endTime={() => this.confirmAnwer(1)}
          />
        </View>
        {this.renderScrollView(children)}
        <View style={styles.viewBottom}>
          {this.renderButtonPrev()}
          {this.renderButton()}
        </View>
        {this.state.loading && (
          <ActivityIndicator
            size="large"
            color="orange"
            style={styles.loading}
          />
        )}
      </View>
    );
  };
}
function mapStateToProps(state) {
  return {
    data: state.reducersAccepted,
    // reducerComponents: state.reducerComponents,
    submitData: state.submitData,
    submitAnswer: state.submitAnswer,
  };
}
export default connect(mapStateToProps)(OnboardingScreens);

const styles = StyleSheet.create({
  // Set width and height to the screen size
  fullScreen: {
    width: width,
    minHeight: height,
  },
  // Main container
  container: {
    backgroundColor: '#fff',
    position: 'relative',
  },
  // Slide
  slide: {
    backgroundColor: 'transparent',
  },
  // Pagination indicators
  question: {
    paddingHorizontal: 20,
    position: 'absolute',
    bottom: 0,
    left: 0,
    alignItems: 'flex-start',
    backgroundColor: '#fff',
  },
  countDown: {
    padding: 10,
    position: 'absolute',
    top: '3%',
    left: 0,
    right: 0,
    flex: 1,
    alignItems: 'center',
    backgroundColor: 'transparent',
  },
  // Pagination dot
  dot: {
    backgroundColor: 'rgba(0,0,0,.25)',
    width: 6,
    height: 8,
    borderRadius: 4,
    marginLeft: 3,
    marginRight: 3,
    marginTop: 3,
    marginBottom: 3,
  },
  // Active dot
  activeDot: {
    backgroundColor: '#FFFFFF',
  },
  // Button wrapper
  buttonWrapper: {
    backgroundColor: 'transparent',
    flexDirection: 'column',
  },
  viewBottom: {
    flex: 0.15,
    flexDirection: 'row',
    justifyContent: 'space-between',
    alignItems: 'center',
    paddingHorizontal: 20,
  },
  buttonPrev: {
    backgroundColor: 'transparent',
    flexDirection: 'column',
  },
  questionText: {
    fontSize: 18,
    fontWeight: 'bold',
    color: 'gray',
  },
  loading: {
    position: 'absolute',
    top: 0,
    left: 0,
    right: 0,
    bottom: 0,
    backgroundColor: 'rgba(0,0,0,0.4)',
  },
});
